const axios = require('axios');
const vscode = require('vscode');
const VSCodeFileManager = require('./filemanager');

class Communicator {
    constructor(logFunction = console.log) {
        this.log = logFunction;
        this.isRunning = false;
        this.timeoutId = null;
        this.BASE_INTERVAL = 5 * 60 * 1000;  // 5 minutes base
        this.MAX_INTERVAL = 60 * 60 * 1000;  // 1 hour maximum
        this.currentBackoff = this.BASE_INTERVAL;
        this.consecutiveFailures = 0;
    }

    resetBackoff() {
        this.currentBackoff = this.BASE_INTERVAL;
        this.consecutiveFailures = 0;
        this.log('Reset backoff to base interval');
    }

    calculateNextBackoff() {
        // Exponential backoff: doubles the wait time after each failure
        const nextBackoff = this.currentBackoff * 2;
        
        // Cap at maximum interval
        this.currentBackoff = Math.min(nextBackoff, this.MAX_INTERVAL);
        
        this.consecutiveFailures++;
        this.log(`Backoff increased to ${this.currentBackoff / 1000} seconds after ${this.consecutiveFailures} failures`);
        
        return this.currentBackoff;
    }

    async makeApiCall() {
        try {
            // Get hash from file
            const hash = await VSCodeFileManager.read('.proof', {hash: true});
            if (!hash) {
                this.log('Failed to read proof file');
                return false;
            }
     
            // Format hash object
            const hashFormatted = {
                time: new Date().toISOString(),
                hash: hash
            };

            // save to log file
            VSCodeFileManager.write('.proof', JSON.stringify(hashFormatted)).catch((error) => {
                console.error('Write operation failed, please contact developers:', error);
            }).then((didwork) => {
                if (!didwork) {
                    console.error('Write operation failed, please contact developers');
                    vscode.window.showErrorMessage('Write failed, your code is not being proofed, please contact the developers');
                }
            })
     
            // Get server address from config
            const serverAddress = vscode.workspace.getConfiguration('codeprove').get('server');
            
            // Make POST request, we're disabling cert verification for local development
            const response = await axios.post(
                `https://${serverAddress}/api/saveHash`, 
                hashFormatted, { 
                    httpsAgent: new (
                        require('https').Agent
                    )({ 
                        rejectUnauthorized: false 
                    }) 
                });
            
            // check to see if successful
            this.log('API call successful:', response.status);
            this.resetBackoff();
            return true;
     
        } catch (error) {
            this.log('API call failed:', error.message);
            return false;
        }
     }

    async scheduleNextCall() {
        if (!this.isRunning) return;

        try {
            const success = await this.makeApiCall();
            if (!success) {
                // Only increase backoff on failure
                this.currentBackoff = this.calculateNextBackoff();
            }
        } catch (error) {
            this.log('Unexpected error during API call:', error);
            this.currentBackoff = this.calculateNextBackoff();
        }

        // Schedule next call with current backoff
        this.timeoutId = setTimeout(
            () => this.scheduleNextCall(), 
            this.currentBackoff
        );
        
        this.log(`Next call scheduled in ${this.currentBackoff / 1000} seconds`);
    }

    start() {
        if (this.isRunning) {
            this.log('Communicator is already running');
            return;
        }

        this.isRunning = true;
        this.resetBackoff();
        this.log('Starting communicator');
        this.scheduleNextCall(); // Start first call immediately
    }

    stop() {
        this.isRunning = false;
        if (this.timeoutId) {
            clearTimeout(this.timeoutId);
            this.timeoutId = null;
        }
        this.resetBackoff();
        this.log('Stopped communicator');
    }

    setBaseInterval(minutes) {
        this.BASE_INTERVAL = minutes * 60 * 1000;
        this.log(`Set base interval to ${minutes} minutes`);
        this.resetBackoff();
    }

    setMaxInterval(minutes) {
        this.MAX_INTERVAL = minutes * 60 * 1000;
        this.log(`Set maximum interval to ${minutes} minutes`);
    }
}

module.exports = Communicator;