const vscode = require('vscode');
const path = require('path');
const fs = require('fs');
const crypto = require('crypto');

class VSCodeFileManager {
    static vscodePath = null;
    static activeReaders = 0;
    static activeWriter = false;
    static writeQueue = [];
    static readQueue = [];

    static init() {
        try {
            const workspaceFolders = vscode.workspace.workspaceFolders;
            if (!workspaceFolders) {
                return false;
            }

            const vscodePath = path.join(workspaceFolders[0].uri.fsPath, '.vscode');
            if (!fs.existsSync(vscodePath)) {
                fs.mkdirSync(vscodePath);
            }
            
            this.vscodePath = vscodePath;
            return true;
        } catch (error) {
            console.error('Failed to initialize .vscode directory:', error);
            return false;
        }
    }

    static async acquireReadLock() {
        return new Promise((resolve) => {
            const tryRead = () => {
                if (!this.activeWriter && this.writeQueue.length === 0) {
                    this.activeReaders++;
                    resolve();
                } else {
                    this.readQueue.push(tryRead);
                }
            };
            tryRead();
        });
    }

    static async acquireWriteLock() {
        return new Promise((resolve) => {
            if (!this.activeWriter && this.activeReaders === 0) {
                this.activeWriter = true;
                resolve();
            } else {
                this.writeQueue.push(resolve);
            }
        });
    }

    static releaseReadLock() {
        this.activeReaders--;
        if (this.activeReaders === 0 && this.writeQueue.length > 0) {
            this.activeWriter = true;
            const nextWriter = this.writeQueue.shift();
            nextWriter();
        }
    }

    static releaseWriteLock() {
        this.activeWriter = false;
        if (this.readQueue.length > 0) {
            const readers = [...this.readQueue];
            this.readQueue = [];
            readers.forEach(reader => reader());
        } else if (this.writeQueue.length > 0) {
            this.activeWriter = true;
            const nextWriter = this.writeQueue.shift();
            nextWriter();
        }
    }

    static async write(filename, content) {
        await this.acquireWriteLock();
        
        try {
            if (!this.vscodePath && !this.init()) {
                return false;
            }

            fs.writeFileSync(path.join(this.vscodePath, filename), content+"\n", { encoding: 'utf8', flag: 'a' });
            return true;
        } catch (error) {
            try {
                if (this.init()) {
                    fs.writeFileSync(path.join(this.vscodePath, filename), content+"\n", { encoding: 'utf8', flag: 'a' });
                    return true;
                }
            } catch (error2){
                console.log(JSON.stringify(error))
                console.log(JSON.stringify(error2))
                return false;
            }
            return false;
        } finally {
            this.releaseWriteLock();
        }
    }

    static async read(filename, { hash = false } = {}) {
        await this.acquireReadLock();
        
        try {
            if (!this.vscodePath && !this.init()) {
                return null;
            }

            const filePath = path.join(this.vscodePath, filename);
            if (!fs.existsSync(filePath)) {
                return null;
            }

            if (hash) {
                return new Promise((resolve, reject) => {
                    const hash = crypto.createHash('sha256');
                    const stream = fs.createReadStream(filePath);
                    
                    stream.on('data', data => {
                        hash.update(data);
                    });
                    
                    stream.on('end', () => {
                        resolve(hash.digest('hex'));
                    });
                    
                    stream.on('error', error => {
                        reject(error);
                    });
                });
            }

            // For non-hash reads, create a readable stream
            return fs.createReadStream(filePath, {
                encoding: 'utf8',
                highWaterMark: 64 * 1024 // 64KB chunks
            });

        } catch (error) {
            try {
                if (this.init()) {
                    const filePath = path.join(this.vscodePath, filename);
                    if (fs.existsSync(filePath)) {
                        if (hash) {
                            return new Promise((resolve, reject) => {
                                const hash = crypto.createHash('sha256');
                                const stream = fs.createReadStream(filePath);
                                stream.on('data', data => hash.update(data));
                                stream.on('end', () => resolve(hash.digest('hex')));
                                stream.on('error', error => reject(error));
                            });
                        }
                        return fs.createReadStream(filePath, {
                            encoding: 'utf8',
                            highWaterMark: 64 * 1024
                        });
                    }
                }
            } catch (error2) {
                console.log(JSON.stringify(error))
                console.log(JSON.stringify(error2))
                return null;
            }
            return null;
        } finally {
            this.releaseReadLock();
        }
    }
}

module.exports = VSCodeFileManager;